# Launch SBM analyze for maximum 2 classes on distance matrices of single OTUs of a sample


# ARGUMENTS
# arg1: type_filename, file with type of OTU -1:not considered, 1:single, 2:composed
# arg2: distance_files_filename, file with the names of distances files in sample
# arg3: sample, directory where are distance matrices
# arg4: save_dir, directory where to save results files 

# RESULT FILES
#         . for each distance matrix of OTU k, the  mean distances between classes  sbm_lambda_<k>.txt file

# Example:  Rscript RUN_SBM.R type_filename.txt distance_files_filename.txt ../distance_dir ../output_dir

library(blockmodels)


# Read script arguments
args <- commandArgs(trailingOnly = TRUE)
type_filename <- as.character(args[1])
distance_files_filename <- as.character(args[2])
sample <- as.character(args[3])
save_dir <- as.character(args[4])

print(type_filename)
print(distance_files_filename)
print(sample)
print(save_dir)

# extract parameters of a SBM model k
get_SBM_params <- function(my_modelp, k) {
  res <- list()
  matricepred <- my_modelp$memberships[[k]]$Z
  res$alpha <- apply(matricepred, 2, mean) # for the update formula of E step in EM.
  res$lambda <- my_modelp$model_parameters[[k]]$lambda
  n <- dim(my_modelp$adj)[1] # number of sequences in the distance matrix
  res$classif <- c(); for (i in 1:n){res$classif[i] <- which.max(matricepred[i,])}
  return(res)
}

kmax <- 2 # maximum 2 classes
typeOTU <- read.csv(type_filename, header=FALSE )$V1
distance_files <- read.csv(distance_files_filename, header=FALSE )$V1
for (k in which(typeOTU==0)) {
  dismat <- read.delim(paste(sample,"/", distance_files[k], sep = ""), header=TRUE)
  dismat[,1]<-NULL # suppress row sequence name
  colnames(dismat)<-c() # suppress column sequence name
  dismat <- as.matrix(dismat) # If not isSymetric is FALSE
  #print(dim(dismat))
  
  my_modelp <- blockmodels::BM_poisson("SBM_sym" ,
                                       dismat,
                                       explore_min = kmax,
                                       explore_max = kmax,
                                       plotting = '',  # no control plot of ICL values
                                       verbosity = 0)
  my_modelp$estimate()

  res <- get_SBM_params(my_modelp, kmax)
  write.table(res$lambda, file = paste(save_dir, "/", as.character(distance_files[k]), ".txt", sep = ""),
              row.names = FALSE, col.names = FALSE)
  }

